<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: TestOfDalTableCoreMySql.php 1610 2005-02-21 15:18:29Z andrew $
*/

/**
 * @package    MaxDal
 * @subpackage TestSuite
 * @author     Andrew Hill <andrew@awarez.net>
 */

require_once MAX_PATH . '/lib/Max.php';
require_once MAX_PATH . '/lib/max/DB.php';
require_once MAX_PATH . '/lib/max/Table/Core.php';
require_once 'Date.php';

/**
 * A class for testing the MAX_Table_Core class.
 */
class MAX_TestOfMaxTableCore extends UnitTestCase
{
    
    /**
     * The constructor method.
     */
    function MAX_TestOfMaxTableCore()
    {        
        $this->UnitTestCase();
    }
    
    /**
     * Tests that the singleton() method only ever returns one
     * class instance.
     */
    function testSingleton()
    {
        MAX::debug('Running test', PEAR_LOG_INFO);
        $first =& MAX_Table_Core::singleton('mysql');
        $second =& MAX_Table_Core::singleton('mysql');
        $this->assertIdentical($first, $second);
    }
    
    /**
     * Tests creating/dropping all of the core tables.
     */
    function testAllCoreTables()
    {
        MAX::debug('Running test', PEAR_LOG_INFO);
        // Use a reference to $GLOBALS['_MAX']['CONF'] so that the configuration
        // options can be changed while the test is running
        $conf =& $GLOBALS['_MAX']['CONF'];
        $dbh =& MAX_DB::singleton();
        // Tables to test
        $coreTables = array(
            'acls',
            'ad_category_assoc',
            'ad_zone_assoc',
            'affiliates',
            'agency',
            'application_variable',
            'banners',
            'cache',
            'campaigns',
            'campaigns_trackers',
            'clients',
            'data_intermediate_ad',
            'data_intermediate_ad_connection',
            'data_intermediate_ad_variable_value',
            'data_summary_ad_hourly',
            'data_summary_zone_impression_history',
            'images',
            'log_maintenance',
            'log_maintenance_statistics',
            'placement_zone_assoc',
            'preference',
            'session',
            'targetstats',
            'trackers',
            'userlog',
            'variables',
            'zones'
        );
        $splitTables = array(
            'data_raw_ad_click',
            'data_raw_ad_impression',
            'data_raw_ad_request',
            'data_raw_tracker_click',
            'data_raw_tracker_impression',
            'data_raw_tracker_variable_value'
        );
        // Create with split mode off
        $conf['table']['split'] = false;
        $tables = MAX_Table_Core::singleton('mysql', true);
        foreach ($coreTables as $tbl) {        
            $tables->createTable($tbl);
            $query = "SELECT COUNT(*) AS number FROM {$conf['table']['prefix']}{$conf['table'][$tbl]}";
            $row = $dbh->getRow($query);
            if (PEAR::isError($row)) {
                return MAX::raiseError($row, MAX_ERROR_DBFAILURE);
            }
            $this->assertEqual($row['number'], 0);
            $tables->dropTable($conf['table']['prefix'].$conf['table'][$tbl]);
        }
        foreach ($splitTables as $tbl) {        
            $tables->createTable($tbl);
            $query = "SELECT COUNT(*) AS number FROM {$conf['table']['prefix']}{$conf['table'][$tbl]}";
            $row = $dbh->getRow($query);
            if (PEAR::isError($row)) {
                return MAX::raiseError($row, MAX_ERROR_DBFAILURE);
            }
            $this->assertEqual($row['number'], 0);
            $tables->dropTable($conf['table']['prefix'].$conf['table'][$tbl]);
        }
        // Create with split mode off
        $conf['table']['split'] = true;
        $tables = MAX_Table_Core::singleton('mysql', true);
        $now = new Date();
        foreach ($coreTables as $tbl) {        
            $tables->createTable($tbl);
            $query = "SELECT COUNT(*) AS number FROM {$conf['table']['prefix']}{$conf['table'][$tbl]}";
            $row = $dbh->getRow($query);
            if (PEAR::isError($row)) {
                return MAX::raiseError($row, MAX_ERROR_DBFAILURE);
            }
            $this->assertEqual($row['number'], 0);
            $tables->dropTable($conf['table']['prefix'].$conf['table'][$tbl]);
        }
        foreach ($splitTables as $tbl) {        
            $tables->createTable($tbl);
            $query = "SELECT COUNT(*) AS number FROM {$conf['table']['prefix']}{$conf['table'][$tbl]}_" .
                     $now->format('%Y%m%d');
            $row = $dbh->getRow($query);
            if (PEAR::isError($row)) {
                return MAX::raiseError($row, MAX_ERROR_DBFAILURE);
            }
            $this->assertEqual($row['number'], 0);
            $tables->dropTable($conf['table']['prefix'].$conf['table'][$tbl].'_'.$now->format('%Y%m%d'));
        }
    }
    
}

?>
