<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: GeoIP.plugin.php 2421 2005-04-07 16:02:07Z radek $
*/

/**
 * @package    MaxPlugins
 * @subpackage Geotargeting
 * @author     Andrew Hill <andrew@awarez.net>
 * @author     Radek Maciaszek <radek@awarez.net>
 *
 * @TODO Not all MaxMind LLC databases have been tested yet!
 */

require_once MAX_PATH . '/plugins/geotargeting/Geotargeting.php';

/**
 *
 * Class to get GeoTargeting information directly from the MaxMind LLC
 * database file, without having it accessed via the C/mod_apache
 * interface.
 *
 * @static
 */
class Plugins_Geotargeting_GeoIP_GeoIP extends Plugins_Geotargeting 
{
    
    /**
     * Return plugin name
     *
     * @return string A string describing the class.
     */
    function getModuleInfo()
    {
        return 'MaxMind GeoIP';
    }
    
    /**
     * The method to look up the GeoTargeting information, based on
     * the IP address.
     *
     * Will use the IP address stored in $GLOBALS['_MAX']['GEO_IP']
     * as the basis of lookups, if set, otherwise $_SERVER['REMOTE_ADDR'].
     *
     * @return array An array that will contain the results of the
     *               GeoTargeting lookup.
     */
    function getInfo()
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        if (isset($GLOBALS['_MAX']['GEO_IP'])) {
            $ip   = $GLOBALS['_MAX']['GEO_IP'];
        } else {
            $ip = $_SERVER['REMOTE_ADDR'];
        }
        // Use the MaxMind GeoIP Country Database
        if ($conf['geotargeting']['geoipCountryLocation'] == '') {
            // Use the free version of the database distributed with Max
            $conf['geotargeting']['geoipCountryLocation'] =
                MAX_PATH . '/plugins/geotargeting/GeoIP/data/FreeGeoIPCountry.dat';
        }
        if ($conf['geotargeting']['geoipCountryLocation'] != '') {
            include_once MAX_PATH . '/plugins/geotargeting/GeoIP/data/geoip.inc';
            $gi = geoip_open($conf['geotargeting']['geoipCountryLocation'], GEOIP_STANDARD);
            // Optimization - geoip_country_id_by_addr() run only once
            $country_id = geoip_country_id_by_addr($gi,$ip);
			if ($country_id !== false) {
				$result['country_code'] = $gi->GEOIP_COUNTRY_CODES[$country_id];
				$result['country_name'] = $gi->GEOIP_COUNTRY_NAMES[$country_id];
			} else {
				$result['country_code'] = false;
				$result['country_name'] = false;
			}
            geoip_close($gi);
        }
        // Use the MaxMind GeoIP Region Database
        if ($conf['geotargeting']['geoipRegionLocation'] != '') {
            include_once MAX_PATH . '/plugins/geotargeting/GeoIP/data/geoip.inc';
            $gi = geoip_open($conf['geotargeting']['geoipRegionLocation'], GEOIP_STANDARD);
            list($country, $region) = geoip_region_by_addr($gi, $ip);
            if (is_null($result['country_code']) || ($result['country_code'] == '')) {
                $result['country_code'] = $country;
            }
            $result['region'] = $region;
            geoip_close($gi);
        }
        // Use the MaxMind GeoIP City Database
        if ($conf['geotargeting']['geoipCityLocation'] != '') {
            include_once MAX_PATH . '/plugins/geotargeting/GeoIP/data/geoipcity.inc';
            $gi = geoip_open($conf['geotargeting']['geoipCityLocation'], GEOIP_STANDARD);
            $record = geoip_record_by_addr($gi, $ip);
            if (is_null($result['country_code']) || ($result['country_code'] == '')) {
                $result['country_code'] = $record->country_code;
            }
            if (is_null($result['country_name']) || ($result['country_name'] == '')) {
                $result['country_name'] = $record->country_name;
            }
            if (is_null($result['region']) || ($result['region'] == '')) {
                $result['region'] = $record->region;
            }
            $result['city'] = $record->city;
            $result['postal_code'] = $record->postal_code;
            $result['latitude'] = (string) $record->latitude;
            $result['longitude'] = (string) $record->longitude;
            $result['dma_code'] = $record->dma_code;
            $result['area_code'] = $record->area_code;
            geoip_close($gi);
        }
        // Use the MaxMind GeoIP Area Code Database
        if ($conf['geotargeting']['geoipAreaLocation'] != '') {
            include_once MAX_PATH . '/plugins/geotargeting/GeoIP/data/geoipcity.inc';
            $gi = geoip_open($conf['geotargeting']['geoipAreaLocation'], GEOIP_STANDARD);
            $record = geoip_record_by_addr($gi, $ip);
            if (is_null($result['country_code']) || ($result['country_code'] == '')) {
                $result['country_code'] = $record->country_code;
            }
            if (is_null($result['country_name']) || ($result['country_name'] == '')) {
                $result['country_name'] = $record->country_name;
            }
            if (is_null($result['region']) || ($result['region'] == '')) {
                $result['region'] = $record->region;
            }
            if (is_null($result['area_code']) || ($result['area_code'] == '')) {
                $result['area_code'] = $record->area_code;
            }
            geoip_close($gi);
        }
        // Use the MaxMind GeoIP DMA Code Database
        if ($conf['geotargeting']['geoipDmaLocation'] != '') {
            include_once MAX_PATH . '/plugins/geotargeting/GeoIP/data/geoipcity.inc';
            $gi = geoip_open($conf['geotargeting']['geoipDmaLocation'], GEOIP_STANDARD);
            $record = geoip_record_by_addr($gi, $ip);
            if (is_null($result['country_code']) || ($result['country_code'] == '')) {
                $result['country_code'] = $record->country_code;
            }
            if (is_null($result['country_name']) || ($result['country_name'] == '')) {
                $result['country_name'] = $record->country_name;
            }
            if (is_null($result['region']) || ($result['region'] == '')) {
                $result['region'] = $record->region;
            }
            if (is_null($result['dma_code']) || ($result['dma_code'] == '')) {
                $result['dma_code'] = $record->dma_code;
            }
            geoip_close($gi);
        }
        // Use the MaxMind GeoIP Organization
        if ($conf['geotargeting']['geoipOrgLocation'] != '') {
            include_once MAX_PATH . '/plugins/geotargeting/GeoIP/data/geoip.inc';
            $gi = geoip_open($conf['geotargeting']['geoipOrgLocation'], GEOIP_STANDARD);
            $result['organisation'] = geoip_org_by_addr($gi, $ip);
            geoip_close($gi);
        }
        // Use the MaxMind GeoIP ISP
        if ($conf['geotargeting']['geoipIspLocation'] != '') {
            include_once MAX_PATH . '/plugins/geotargeting/GeoIP/data/geoip.inc';
            $gi = geoip_open($conf['geotargeting']['geoipIspLocation'], GEOIP_STANDARD);
            $result['isp'] = geoip_org_by_addr($gi, $ip);
            geoip_close($gi);
        }
        // Use the MaxMind GeoIP Netspeed
        if ($conf['geotargeting']['geoipNetspeedLocation'] != '') {
            require_once MAX_PATH . '/plugins/geotargeting/GeoIP/data/geoip.inc';
            $gi = geoip_open($conf['geotargeting']['geoipNetspeedLocation'], GEOIP_STANDARD);
            $netspeed = geoip_country_id_by_addr($gi, $ip);
            if ($netspeed == GEOIP_UNKNOWN_SPEED) {
                $result['netspeed'] = "Unknown";
            } else if ($netspeed == GEOIP_DIALUP_SPEED) {
                $result['netspeed'] = "Dailup";
            } else if ($netspeed == GEOIP_CABLEDSL_SPEED) {
                $result['netspeed'] = "Cable/DSL";
            } else if ($netspeed == GEOIP_CORPORATE_SPEED) {
                $result['netspeed'] = "Corporate";
            }
            geoip_close($gi);
        }
        // Return the result(s)
        return $result;
    }

}

?>